package de.mattv.fileserver;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.NonNull;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestClient;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.support.DefaultHandlerExceptionResolver;

import java.io.PrintWriter;
import java.io.StringWriter;

@Component
public class ExceptionHandler extends DefaultHandlerExceptionResolver {
    private record GotifyMessage(String title, String message) {}

    @Value("${gotify.url:}")
    private String gotifyUrl;

    @Value("${gotify.token:}")
    private String gotifyToken;

    private RestClient restClient = null;

    @Override
    protected ModelAndView doResolveException(@NonNull HttpServletRequest request, @NonNull HttpServletResponse response, Object handler, @NonNull Exception ex) {
        if (!gotifyToken.isEmpty() && !gotifyUrl.isEmpty()) {
            if (restClient == null)
                restClient = RestClient.builder().baseUrl(gotifyUrl).build();

            StringWriter writer = new StringWriter();
            PrintWriter printWriter = new PrintWriter(writer);
            ex.printStackTrace(printWriter);
            printWriter.flush();
            restClient.post()
                    .uri("/message?token=" + gotifyToken)
                    .body(new GotifyMessage("Encountered an error", writer.toString()))
                    .retrieve();
        }
        return super.doResolveException(request, response, handler, ex);
    }
}
