package de.mattv.fileserver;

import de.mattv.fileserver.util.NonNull;
import jakarta.servlet.RequestDispatcher;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.boot.autoconfigure.http.HttpMessageConverters;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.context.annotation.Bean;
import org.springframework.http.HttpOutputMessage;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.http.converter.HttpMessageNotWritableException;
import org.springframework.http.converter.StringHttpMessageConverter;
import org.springframework.http.converter.json.MappingJackson2HttpMessageConverter;
import jakarta.servlet.Filter;

import java.io.IOException;
import java.lang.reflect.Type;
import java.util.List;

@SpringBootApplication
public class FileServerApplication {
    public static void main(String[] args) {
        SpringApplication.run(FileServerApplication.class, args);
    }

    public static class MessageConverter extends MappingJackson2HttpMessageConverter {
        @Override
        protected void writeInternal(@NonNull @lombok.NonNull Object object, Type type, @NonNull HttpOutputMessage outputMessage)
                throws IOException, HttpMessageNotWritableException {
            byte[] data = getObjectMapper().writeValueAsBytes(object);
            outputMessage.getHeaders().add("Content-Length", String.valueOf(data.length));
            outputMessage.getBody().write(data);
        }
    }

    @Bean
    public HttpMessageConverters createHttpMessageConverter() {
        return new HttpMessageConverters(new MessageConverter()) {
            @Override
            protected List<HttpMessageConverter<?>> postProcessConverters(List<HttpMessageConverter<?>> converters) {
                return converters.stream().filter(v -> !v.getClass().equals(StringHttpMessageConverter.class)).toList();
            }

            @Override
            protected List<HttpMessageConverter<?>> postProcessPartConverters(List<HttpMessageConverter<?>> converters) {
                return converters.stream().filter(v -> !v.getClass().equals(StringHttpMessageConverter.class)).toList();
            }
        };
    }

    @Bean
    public FilterRegistrationBean<Filter> nonApiToIndex() {
        return new FilterRegistrationBean<>((servletRequest, servletResponse, chain) -> {
            String uri = ((HttpServletRequest) servletRequest).getRequestURI();
            if (uri.startsWith("/api")) {
                chain.doFilter(servletRequest, servletResponse);
            } else if (uri.equals("/")) {
                RequestDispatcher requestDispatcher = servletRequest.getRequestDispatcher("/index.html");
                requestDispatcher.forward(servletRequest, servletResponse);
            } else {
                ((HttpServletResponse) servletResponse).sendRedirect("/");
            }
        });
    }
}
