package de.mattv.fileserver.routes.auth;

import de.mattv.fileserver.data.Data;
import de.mattv.fileserver.data.User;
import de.mattv.fileserver.security.TokenService;
import de.mattv.fileserver.util.AuthUser;
import de.mattv.fileserver.util.AutoCloseLock;
import de.mattv.fileserver.util.UserRestController;
import de.mattv.fileserver.util.NonNull;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Optional;

@UserRestController
public class ChangePW {
    private record Body(@NonNull String oldPassword, @NonNull String newPassword) {}

    @PostMapping("/auth/change_password")
    private @NonNull Optional<String> change(@AuthUser User user, @RequestBody Body body) {
        if (body.newPassword.length() < 6)
            return Optional.of("Password must be at least 6 characters");

        try (AutoCloseLock ignored = new AutoCloseLock(Data.USER_LOCK.readLock())) {
            if (!AuthUtils.PW_ENCODER.matches(body.oldPassword, user.password))
                return Optional.of("Old password is incorrect");

            user.password = AuthUtils.PW_ENCODER.encode(body.newPassword);
            Data.save();
            TokenService.logoutAll(user.id);
            return Optional.empty();
        }
    }
}
